@@ -70,7 +70,7 @@
  *
  *    // Print some infos about the formatter GESAsset
  *    for (tmp = formatter_assets; tmp; tmp = tmp->next) {
- *      g_print ("Name of the formatter: %s, file extension it produces: %s",
+ *      gst_print ("Name of the formatter: %s, file extension it produces: %s",
  *        ges_meta_container_get_string (
  *          GES_META_CONTAINER (tmp->data), GES_META_FORMATTER_NAME),
  *        ges_meta_container_get_string (
@@ -351,34 +351,27 @@ ges_asset_extract_default (GESAsset * asset, GError ** error)
   GParameter *params;
   GESAssetPrivate *priv = asset->priv;
   GESExtractable *n_extractable;
-
+  gint i;
+  GValue *values;
+  const gchar **names;
 
   params = ges_extractable_type_get_parameters_from_id (priv->extractable_type,
       priv->id, &n_params);
 
-#if GLIB_CHECK_VERSION(2, 53, 1)
-  {
-    gint i;
-    GValue *values;
-    const gchar **names;
 
-    values = g_malloc0 (sizeof (GValue) * n_params);
-    names = g_malloc0 (sizeof (gchar *) * n_params);
+  values = g_malloc0 (sizeof (GValue) * n_params);
+  names = g_malloc0 (sizeof (gchar *) * n_params);
 
-    for (i = 0; i < n_params; i++) {
-      values[i] = params[i].value;
-      names[i] = params[i].name;
-    }
-
-    n_extractable =
-        GES_EXTRACTABLE (g_object_new_with_properties (priv->extractable_type,
-            n_params, names, values));
-    g_free (names);
-    g_free (values);
+  for (i = 0; i < n_params; i++) {
+    values[i] = params[i].value;
+    names[i] = params[i].name;
   }
-#else
-  n_extractable = g_object_newv (priv->extractable_type, n_params, params);
-#endif
+
+  n_extractable =
+      GES_EXTRACTABLE (g_object_new_with_properties (priv->extractable_type,
+          n_params, names, values));
+  g_free (names);
+  g_free (values);
 
   while (n_params--)
     g_value_unset (&params[n_params].value);
@@ -451,7 +444,7 @@ ges_asset_finalize (GObject * object)
 {
   GESAssetPrivate *priv = GES_ASSET (object)->priv;
 
-  GST_DEBUG_OBJECT (object, "finalizing");
+  GST_LOG_OBJECT (object, "finalizing");
 
   if (priv->id)
     g_free (priv->id);
@@ -1371,10 +1364,10 @@ ges_asset_request (GType extractable_type, const gchar * id, GError ** error)
  *
  *   asset = ges_asset_request_finish (res, &error);
  *   if (asset) {
- *    g_print ("The file: %s is usable as a GESUriClip",
+ *    gst_print ("The file: %s is usable as a GESUriClip",
  *        ges_asset_get_id (asset));
  *   } else {
- *    g_print ("The file: %s is *not* usable as a GESUriClip because: %s",
+ *    gst_print ("The file: %s is *not* usable as a GESUriClip because: %s",
  *        ges_asset_get_id (source), error->message);
  *   }
  *
